from .list_files_in_container import list_files_in_container
from .read_buffer import read_buffer_default_credential, read_buffer_connection_string
from .read_parquet_from_container_default_credential import read_parquet_from_container_default_credential
from .read_parquet import read_parquet_from_container
from .upload_dataframe_to_parquet import upload_pandas_dataframe_to_parquet
from .write_buffer import write_buffer_default_credential, write_buffer_connection_string
from .write_parquet_to_container_default_credential import write_parquet_to_container_default_credential




# Note the need to login to azure to authenticate: this allows a connection to blob storage
# Do this by running the following in the console linked to your python kernel:
# az login

from azure.identity import DefaultAzureCredential
from azure.storage.blob import BlobServiceClient
from io import BytesIO

def read_buffer_default_credential(blob_name: str, container_name: str, account_url: str):

    ''' Returns a buffer: a bytes object which can be read, eg to read a csv file from blob storage:
    
    buffer = read_parquet_from_container_default_credential(....)
    pd.read_csv(buffer)
     '''
    credential = DefaultAzureCredential()

    blob_service_client = BlobServiceClient(account_url, credential=credential)
    
    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    stream = blob_client.download_blob()

    buffer = BytesIO(stream.readall())
    return buffer


def read_buffer_connection_string(
    blob_name: str,
    container_name: str,
    connection_string: str,
)e:
    '''
    Returns a buffer: a bytes object which can be read

    Authenticates with a connection string

    # Example
    connection_string = 'LONG AND SECRET CONNECTION STRING'container_name = "gold-curated"
    blob_name = "bloomberg_daily/parquet/bloomberg_daily_v4.parquet"

    buffer = read_parquet_from_container(
        blob_name,
        container_name,
        connection_string,
    )

    # Load datafrane
    df = pd.read_parquet(buffer)
    print(df.shape)
    '''

    blob_service_client = BlobServiceClient.from_connection_string(connection_string)
    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    # Download the blob data into a stream
    stream = blob_client.download_blob()

    # Read the stream into a buffer
    buffer = BytesIO(stream.readall())
    return buffer






# Note the need to login to azure to authenticate: this allows a connection to blob storage
# Do this by running the following in the console linked to your python kernel:
# az login

from azure.identity import DefaultAzureCredential
from azure.storage.blob import BlobServiceClient  
import pandas as pd

def write_parquet_to_container_default_credential(df, blob_name, container_name, account_url):
    
    credential = DefaultAzureCredential()
    blob_service_client = BlobServiceClient(account_url, credential=credential)

    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    buffer = BytesIO()
    df.to_parquet(buffer)

    blob_client.upload_blob(buffer.getvalue(), overwrite = True)

# Example usage
#df = pd.DataFrame({"col1": [1,2,3,4]}) 
#write_parquet_to_container(df, "output.parquet", container_name, account_url)



from azure.storage.blob import BlobServiceClient

def list_files_in_container(
    connection_string: str,
    container_name: str,
):
    '''
    # Example of using func
    connection_string = 'LONG AND SECRET CONNECTION STRING'
    container_name = 'test'
    list_files_in_container(connection_string, container_name)
    '''
    blob_service_client = BlobServiceClient.from_connection_string(connection_string)
    container_client = blob_service_client.get_container_client(container_name)
    blob_list = container_client.list_blobs()

    return [blob.name for blob in blob_list]






from azure.storage.blob import BlobServiceClient
import pandas as pd
from io import BytesIO


def read_parquet_from_container(
    blob_name: str,
    container_name: str,
    connection_string: str,
) -> pd.DataFrame:
    '''
    # Example
    connection_string = 'LONG AND SECRET CONNECTION STRING'container_name = "gold-curated"
    blob_name = "bloomberg_daily/parquet/bloomberg_daily_v4.parquet"


    original_daily = read_parquet_from_container(
        blob_name,
        container_name,
        connection_string,
    )

    # Show the DataFrame
    print(original_daily.shape)
    '''

    blob_service_client = BlobServiceClient.from_connection_string(connection_string)
    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    # Download the blob data into a stream
    stream = blob_client.download_blob()

    # Read the stream into a DataFrame
    buffer = BytesIO(stream.readall())
    return pd.read_parquet(buffer)







# Note the need to login to azure to authenticate: this allows a connection to blob storage
# Do this by running the following in the console linked to your python kernel:
# az login

from azure.identity import DefaultAzureCredential
from azure.storage.blob import BlobServiceClient  
import pandas as pd

def write_buffer_default_credential(buffer, blob_name, container_name, account_url):
    '''
    Writes a buffer (a store of bytes in memory), which you should have already made and written to, to blob storage

    # Example usage: note you initialise the buffer then write the datafrane to it
    buffer = BytesIO()
    df.to_parquet(buffer, index=False, engine='pyarrow')
    write_buffer_default_credential(buffer, ...)
    '''
    
    credential = DefaultAzureCredential()
    blob_service_client = BlobServiceClient(account_url, credential=credential)

    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    # Go to the start of the BytesIO object
    buffer.seek(0)

    blob_client.upload_blob(buffer.getvalue(), overwrite = True)



def write_buffer_connection_string(
    buffer,
    blob_name: str,
    container_name: str,
    connection_string: str
) -> None:

    """
    Writes a buffer (a store of bytes in memory), which you should have already made and written to, to blob storage

    Authenticates with a connection string

    # Usage example:
    buffer = BytesIO()
    df.to_parquet(buffer, index=False, engine='pyarrow')
    
    connection_string = 'LONG AND SECRET CONNECTION STRING'
    container_name = "gold-curated"
    blob_name = "bloomberg_daily/parquet/test.parquet"
    write_buffer_connection_string(buffer, 'test.parquet', container_name, connection_string)
    """

    # Create a BlobServiceClient using the Azure connection string
    blob_service_client = BlobServiceClient.from_connection_string(connection_string)

    # Get a BlobClient object representing the named blob within the named container
    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    # Go to the start of the BytesIO object
    buffer.seek(0)

    # Create the blob in Azure Blob Storage using the data in buffer
    blob_client.upload_blob(buffer.read(), overwrite=True)





from azure.storage.blob import BlobServiceClient
import pandas as pd
from io import BytesIO

def upload_pandas_dataframe_to_parquet(
    df: pd.DataFrame,
    blob_name: str,
    container_name: str,
    connection_string: str
) -> None:

    """
    # Usage example:
    df = pd.DataFrame({'column1': [1, 2, 3], 'column2': ['a', 'b', 'c']})
    connection_string = 'LONG AND SECRET CONNECTION STRING'
    container_name = "gold-curated"
    blob_name = "bloomberg_daily/parquet/test.parquet"
    upload_pandas_dataframe_to_parquet(df, 'test.parquet', container_name, connection_string)
    """

    # Create a BlobServiceClient using the Azure connection string
    blob_service_client = BlobServiceClient.from_connection_string(connection_string)

    # Get a BlobClient object representing the named blob within the named container
    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    # Use Pandas DataFrame to_parquet method to write DataFrame to BytesIO object
    buffer = BytesIO()
    df.to_parquet(buffer, index=False, engine='pyarrow')

    # Go to the start of the BytesIO object
    buffer.seek(0)

    # Create the blob in Azure Blob Storage using the data in buffer
    blob_client.upload_blob(buffer.read(), overwrite=True)





# Note the need to login to azure to authenticate: this allows a connection to blob storage
# Do this by running the following in the console linked to your python kernel:
# az login

from azure.identity import DefaultAzureCredential
from azure.storage.blob import BlobServiceClient
import pandas as pd
from io import BytesIO

def read_parquet_from_container_default_credential(blob_name: str, container_name: str, account_url: str) -> pl.DataFrame:

    credential = DefaultAzureCredential()

    blob_service_client = BlobServiceClient(account_url, credential=credential)
    
    blob_client = blob_service_client.get_blob_client(container=container_name, blob=blob_name)

    stream = blob_client.download_blob()

    buffer = BytesIO(stream.readall())
    return pd.read_parquet(buffer)

#account_url = "https://hmtdatasciencedevrga07f.blob.core.windows.net/"
#read_parquet_from_container('output.parquet', container_name, account_url)



from setuptools import setup, find_packages

# Read requirements.txt
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name="hmt_azure_funcs",
    version="0.5",
    packages=find_packages(),
    install_requires=requirements,
    author="hmt data science team",
    description="code to interact with azure",
)